/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/
/*!
 ***************************************************************************
 * \file
 *    q_around.h
 *
 * \author
 *    Alexis Michael Tourapis
 *
 * \brief
 *    Headerfile for Quantization Adaptive Rounding
 **************************************************************************
 */

#ifndef _Q_AROUND_H_
#define _Q_AROUND_H_

extern int   **fadjust8x8, **fadjust4x4, ***fadjust4x4Cr, ***fadjust8x8Cr;

typedef struct around_offset
{
  int  **InterFAdjust4x4;
  int  **IntraFAdjust4x4;
  int  **InterFAdjust8x8; 
  int  **IntraFAdjust8x8;
  int ***InterFAdjust4x4Cr;
  int ***IntraFAdjust4x4Cr;
  int ***InterFAdjust8x8Cr;
  int ***IntraFAdjust8x8Cr;
} ARoundOffset;

// Create / Clear adaptive rounding variables
void setup_adaptive_rounding (InputParameters *params);
void clear_adaptive_rounding (InputParameters *params);

void store_adaptive_rounding_parameters        (Macroblock *currMB, int mode);
void store_adaptive_rounding_parameters_luma   (Macroblock *currMB, int mode);
void store_adaptive_rounding_parameters_chroma (Macroblock *currMB, int mode);

void store_adaptive_rounding (ImageParameters *img, int block_y, int block_x);
void update_adaptive_rounding(ImageParameters *img, int block_y, int block_x);
void update_offset_params    (Macroblock *currMB, int mode, int luma_transform_size_8x8_flag);

#endif

