/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
************************************************************************
* \file quant_cr.h
*
* \brief
*    Quantization process for chroma header file
*
* \author
*    Alexis Michael Tourapis         <alexismt@ieee.org>                
*
************************************************************************
*/

#ifndef _QUANT_CR_H_
#define _QUANT_CR_H_

void init_quant_Chroma(InputParameters *params, ImageParameters *img);

int (*quant_4x4cr)   (int (*tblock)[16], int block_y, int block_x, int qp, 
                      int*  ACLevel, int*  ACRun, 
                      int **fadjust4x4, int **levelscale, int **invlevelscale, int **leveloffset,
                      int *coeff_cost, const byte (*pos_scan)[2], const byte *c_cost);

int quant_dc2x2_normal (int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust2x2, int levelscale, int invlevelscale, int **leveloffset,
                        const byte (*pos_scan)[2]);

int quant_dc2x2_around (int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust2x2, int levelscale, int invlevelscale, int **leveloffset,
                        const byte (*pos_scan)[2]);

int quant_dc2x2_trellis(int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust, int levelscale, int invlevelscale, int **leveloffset, 
                        const byte (*pos_scan)[2]);

int quant_dc4x2_normal (int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust, int levelscale, int invlevelscale, int **leveloffset,
                        const byte (*pos_scan)[2]);

int quant_dc4x2_around (int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust, int levelscale, int invlevelscale, int **leveloffset,
                        const byte (*pos_scan)[2]);

int quant_dc4x2_trellis(int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust, int levelscale, int invlevelscale, int **leveloffset,
                        const byte (*pos_scan)[2]);

int (*quant_dc_cr)     (int (*tblock)[4], int qp, int* DCLevel, int* DCRun, 
                        int **fadjust, int levelscale, int invlevelscale, int **leveloffset,
                        const byte (*pos_scan)[2]);

int (*quant_ac4x4cr)   (int (*tblock)[16], int block_y, int block_x, int qp,                 
                        int*  ACLevel, int*  ACRun, 
                        int **fadjust4x4, int **levelscale, int **invlevelscale, int **leveloffset,
                        int *coeff_cost, const byte (*pos_scan)[2], const byte *c_cost, int type);

#endif

