/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/
/*!
 ***************************************************************************
 * \file
 *    wp_mcprec.h
 *
 * \brief
 *    Headerfile for Improved Motion Compensatation Precision Scheme using Weighted Prediction
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *     - Athanasios Leontaris            <aleon@dolby.com>
 *
 * \date
 *    16 July 2008
 *
 **************************************************************************
 */

#ifndef _WP_MCPREC_H_
#define _WP_MCPREC_H_


#define WP_MCPREC_PLUS0       4
#define WP_MCPREC_PLUS1       5
#define WP_MCPREC_MINUS0      6
#define WP_MCPREC_MINUS1      7
#define WP_MCPREC_MINUS_PLUS0 8
#define WP_REGULAR            9

typedef struct
{
  int PicNum;              // PicNum/FrameNum
  int POCNum;              // POC
}
WeightedPredRefX;

typedef struct
{
  int algorithm;
}
WPXPass;

typedef struct
{
  int               num_wp_ref_list[2];         // num of elements in each of the above matrices [LIST]
  WeightedPredRefX *wp_ref_list[2];             // structure with reordering and WP information for ref frames [LIST]
  WPXPass          *curr_wp_rd_pass;
  WPXPass           wp_rd_passes[3];            // frame_picture [0...4] (MultiRefWeightedPred == 2)
}
WPXObject;

WPXObject *pWPX;

void   wpxInitWPXObject( void );
void   wpxFreeWPXObject( void );
void   wpxInitWPXPasses( InputParameters *params );
void   wpxModifyRefPicList( ImageParameters *img );
// Note that at some point, InputParameters params contents should be copied into ImageParameters *img. 
// This would eliminate need of having to use both structures
int    wpxDetermineWP( InputParameters *params, ImageParameters *img, int clist, int n );
void   wpxAdaptRefNum( ImageParameters *img );
void   wpxReorderLists( ImageParameters *img, Slice *currSlice );

#endif

