/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third-parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004, 2005, 2006, 2007, 2008.
 **********************************************************************/

/*!
 *************************************************************************************
 * \file quantChroma.c
 *
 * \brief
 *    Quantization initialization function for Chroma blocks
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *    - Alexis Michael Tourapis                  <alexismt@ieee.org>
 *
 *************************************************************************************
 */

#include "contributors.h"

#include "global.h"
#include "quant4x4.h"
#include "quantChroma.h"

/*!
************************************************************************
* \brief
*    Quantization initialization function
*
************************************************************************
*/
void init_quant_Chroma(InputParameters *params, ImageParameters *img)
{
  if (params->UseRDOQuant == 1 && params->RDOQ_CR == 1)
  {
    quant_ac4x4cr = quant_ac4x4_trellis;
    if (params->yuv_format == YUV422)
      quant_dc_cr   = quant_dc4x2_trellis;
    else
      quant_dc_cr   = quant_dc2x2_trellis;
  }
  else if (params->UseRDOQuant == 1 || (!img->AdaptiveRounding))
  {
    quant_ac4x4cr = quant_ac4x4_normal;
    if (params->yuv_format == YUV422)
      quant_dc_cr   = quant_dc4x2_normal;
    else
      quant_dc_cr   = quant_dc2x2_normal;
  }
  else
  {
    quant_ac4x4cr = quant_ac4x4_around;
    if (params->yuv_format == YUV422)
      quant_dc_cr   = quant_dc4x2_around;
    else
      quant_dc_cr   = quant_dc2x2_around;
  }
}


